package CASObservatory;

import ij.*;
import ij.process.*;
import ij.util.Tools;
import ij.gui.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.*;

import astroj.AstroCanvas;
import astroj.Centroid;
import astroj.IJU;
import astroj.WCS;
import ij.plugin.*;
import ij.measure.ResultsTable;

public class Object_Catalog implements PlugIn, DialogListener, MouseListener {
	private static int n = 5;
	private static double lineWidth = 2;
	private static double xcenter, ycenter;
	private static double innerR, outerR;
	private boolean measure;
	private boolean hide;
	private ImagePlus img;
	ImageCanvas canvas;
	AstroCanvas ac;
	private double[][] theData = { { 1, 23.3689677964, 60.9626214559 }, { 2, 23.3816793898, 60.9942595044 },
			{ 3, 23.2579989881, 61.0336938592 }, { 4, 23.3170867206, 61.0585397889 },
			{ 5, 23.4353177861, 61.0037680488 }, { 6, 23.5964087508, 60.948884454 },
			{ 7, 23.4303992794, 60.9650781473 }, { 8, 23.4717255151, 60.9466820963 },
			{ 9, 23.4471313637, 60.9433952063 }, { 10, 23.3639362079, 60.9639428832 },
			{ 11, 23.267303615, 61.0770842113 }, { 12, 23.3998953368, 60.9656341206 },
			{ 13, 23.3098524742, 61.061428598 } };

	public double[][] stars() {

		double[][] res = new double[theData.length][2];

		WCS wcs = new WCS(img);
		double data[] = new double[2];

		for (int i = 0; i < theData.length; i++) {
			data[0] = theData[i][1] * 15.0;

			data[1] = theData[i][2];

			double[] tmp = wcs.wcs2pixels(data);
			res[i][0] = tmp[0];
			res[i][1] = tmp[1];

		}

		return res;
	}

	public void run(String arg) {
		if (IJ.versionLessThan("1.47i"))
			return;
		img = IJ.getImage();
		ImageWindow win = img.getWindow();
		canvas = win.getCanvas();
		canvas.addMouseListener(this);
		ac = (AstroCanvas) win.getCanvas();
		double d[] = { 359.63533743167665, 61.32589722262076 };
		WCS wcs = new WCS(img);
		double[] tmp = wcs.wcs2pixels(d);
		System.out.println(">>>>>>>>>" + tmp[0] + "\t" + tmp[1]);

		double[][] ls_stars = stars();
		for (int i = 0; i < ls_stars.length; i++) {

			double x = ls_stars[i][0];
			double y = ls_stars[i][1];
			System.out.println("x" + x + " y" + y);
		}

		Roi roi = img.getRoi();
		Rectangle b = new Rectangle(0, 0, img.getWidth(), img.getHeight());
		boolean isRoi = roi != null && roi.isArea();
		boolean isLine = roi != null && roi.getType() == Roi.LINE;
		if (isRoi || isLine)
			b = roi.getBounds();
		if (outerR == 0f || isRoi || isLine)
			setup(b, isLine);
		if (!IJ.isMacro())
			drawCircles();
		showDialog();
	}

	void drawCircles() {
		if (hide || n == 0)
			img.setOverlay(null);
		else {
			Overlay overlay = new Overlay();
			double[][] ls_stars = stars();

			for (int i = 0; i < ls_stars.length; i++) {

				int r = 5;
				int x = (int) ls_stars[i][0];
				int y = (int) ls_stars[i][1];
				System.out.println("x" + x + " y" + y);
				Roi circle = new OvalRoi(x - r, y - r, 2 * r, 2 * r);
				circle.setStrokeColor(Color.red);
				circle.setStrokeWidth(0.0);
				overlay.add(circle);
			}

			// int r = 5;
			// Roi circle = new OvalRoi(94 - r, 49 - r, 2 * r, 2 * r);
			// circle.setStrokeColor(Color.red);
			// circle.setStrokeWidth(1);
			// overlay.add(circle);
			//
			// circle = new OvalRoi(202, 184, 10, 10);
			// circle.setStrokeColor(Color.red);
			// circle.setStrokeWidth(1);
			// overlay.add(circle);

			img.setOverlay(overlay);
			if (measure)
				measure(overlay);
		}
	}

	Overlay overlay = new Overlay();

//	void drawCircle(int x, int y, int r) {
//
//		Roi circle = new OvalRoi(x - r, y - r, 2 * r, 2 * r);
//		circle.setStrokeColor(Color.blue);
//		circle.setStrokeWidth(0.0);
//		overlay.add(circle);
//
//		img.setOverlay(overlay);
//
//	}

//	void circle(double x, double y, int r) {
//
//		Rectangle rect = a2rect(x, y, r);
//		OvalRoi circle = new OvalRoi(rect.x, rect.y, rect.width, rect.height);
//		circle.setStrokeColor(Color.ORANGE);
//		overlay.add(circle);
//
//		img.setOverlay(overlay);
//
//	}

	void kreis(double x, double y, int r) {

		Roi circle = new OvalRoi(x - r, y - r, r * 2, r * 2);
		circle.setStrokeColor(Color.ORANGE);
		overlay.add(circle);
		img.setOverlay(overlay);
	}

	/**
	 * Help routine to convert real pixel position + aperture radius into
	 * centered pixel rectangle.
	 */
	public static Rectangle a2rect(double x, double y, double r) {
		Rectangle rect = new Rectangle();

		rect.width = (int) (2.0 * r);
		rect.height = rect.width + 1 - (rect.width % 2); // SHOULD BE ODD
		rect.width = rect.height;
		rect.x = (int) (x - Centroid.PIXELCENTER) - rect.width / 2;
		rect.y = (int) (y - Centroid.PIXELCENTER) - rect.height / 2;
		return rect;
	}

	void setup(Rectangle b, boolean isLine) {
		xcenter = b.x + b.width / 2;
		ycenter = b.y + b.height / 2;
		float size = Math.min(b.width, b.height);
		if (isLine)
			size = (float) Math.sqrt(b.width * b.width + b.height * b.height);
		innerR = size / 20f;
		outerR = size / 2f;
	}

	void showDialog() {
		NonBlockingGenericDialog gd = new NonBlockingGenericDialog("Circles");
		// GenericDialog gd = new GenericDialog("Circles");
		gd.addNumericField("Circles:", n, 0);
		gd.addNumericField("Line width:", lineWidth, 1);
		gd.addNumericField("X center:", xcenter, 1);
		gd.addNumericField("Y center:", ycenter, 1);
		gd.addNumericField("Inner radius:", innerR, 1);
		gd.addNumericField("Outer radius:", outerR, 1);
		gd.addCheckbox("Measure", false);
		gd.addCheckbox("Hide", false);
		gd.addDialogListener(this);
		gd.showDialog();
		if (gd.wasCanceled())
			img.setOverlay(null);
	}

	public boolean dialogItemChanged(GenericDialog gd, AWTEvent e) {
		if (gd.wasCanceled())
			return false;
		n = (int) gd.getNextNumber();
		lineWidth = (float) gd.getNextNumber();
		xcenter = (float) gd.getNextNumber();
		ycenter = (float) gd.getNextNumber();
		innerR = (float) gd.getNextNumber();
		outerR = (float) gd.getNextNumber();
		measure = gd.getNextBoolean();
		hide = gd.getNextBoolean();
		drawCircles();
		return true;
	}

	void measure(Overlay overlay) {
		int n = overlay.size();

		ResultsTable rt = ResultsTable.getResultsTable();
		ImageProcessor ip = img.getProcessor();
		for (int i = 0; i < n; i++) {
			Roi circle = overlay.get(i);
			Rectangle2D.Double r = circle.getFloatBounds();
			FloatPolygon p = circle.getInterpolatedPolygon(1, false);

			Double sum = 0.0;
			for (int j = 0; j < p.npoints; j++)
				sum += ip.getPixelValue((int) Math.round(p.xpoints[j]), (int) Math.round(p.ypoints[j]));
			double mean = sum / p.npoints;
			rt.incrementCounter();
			rt.addValue("Radius", r.width / 2);
			rt.addValue("Samples", p.npoints);
			rt.addValue("Mean", mean);

		}
		rt.show("Results");
	}

	@Override
	public void mouseClicked(MouseEvent e) {

		plotInfo();
		starCentroid();
	}

	/**
	 * Algorithm Howell p. 105 image centering
	 */
	public void starCentroid() {

		// step: region rectangle
		Roi roi = img.getRoi();
		Rectangle b = roi.getBounds();
		System.out.println(b.x + " " + b.y + " " + b.height + "  " + b.width);

		int z = b.height;
		int s = b.width;

		double I[] = new double[z];
		double J[] = new double[s];
		ImageProcessor ip = img.getProcessor();

		// zeilen Ii berechnen
		for (int i = 0; i < z; i++)
			for (int j = 0; j < s; j++) {
				I[i] += ip.getPixelValue((int) Math.round(b.x + i), (int) Math.round(b.y + j));
			}
		// spalten Ji berechnen
		for (int j = 0; j < s; j++) {
			for (int i = 0; i < z; i++)
				J[j] += ip.getPixelValue((int) Math.round(b.x + i), (int) Math.round(b.y + j));
		}
		// summen bilden
		double sumI = 0;
		for (int i = 0; i < z; i++) {
			sumI += I[i];
		}
		sumI /= (z);

		double sumJ = 0;
		for (int i = 0; i < s; i++) {
			sumJ += J[i];
		}
		sumJ /= (s);

		// x/y center berechnen
		double valZ = 0, valN = 0;

		for (int i = 0; i < z; i++) {
			double tmp = I[i] - sumI;
			if (tmp > 0) {
				valZ += tmp * (b.x + i);
				valN += tmp;

			}
		}
		double xCenter = valZ / valN;

		valZ = 0;
		valN = 0;
		for (int i = 0; i < s; i++) {
			double tmp = J[i] - sumJ;
			if (tmp > 0) {
				valZ += tmp * (b.y + i);
				valN += tmp;

			}
		}
		double yCenter = valZ / valN;

		//////////////// 7

		xCenter += 0.5;
		yCenter += 0.5;
		// ausgabe
		System.out.println("Center (x/y): " + xCenter + "\t" + yCenter);

		// Rectangle rect = a2rect(xCenter, yCenter, 2);

	//	drawCircle((int) xCenter, (int) yCenter, 2);
	//	circle(xCenter, yCenter, 2);
	//	circle(xCenter + Centroid.PIXELCENTER, yCenter + Centroid.PIXELCENTER, 2);

		int xx = (int) xCenter;
		int yy = (int) yCenter;
		System.out.println("x und y: " + xx + "\t" + yy);		

		kreis(xCenter, yCenter, 2);
	}

	// retrieve current mouse position
	public double[] mousePosition() {
		double res[] = new double[2];
		res[0] = canvas.offScreenXD(canvas.ABORT);
		res[1] = canvas.offScreenYD(canvas.ABORT);
		return res;
	}

	public void plotInfo() {
		/*
		 * beispiel koordinaten imageJ/RA und Fits todo: centroid betrachtung!?
		 */
		// a) imagej koordinaten
		double[] mouse = mousePosition();
		double X0 = mouse[0];
		double Y0 = mouse[1];
		System.out.println("ImageJ X: " + X0 + "\tImageJ Y:" + Y0);

		// drawCircle((int) X0, (int) Y0);
		// b) ra/dec wcs koordinaten
		WCS wcs = new WCS(img);
		double data[] = new double[2];
		data[0] = X0;
		data[1] = Y0;

		double[] res = wcs.pixels2wcs(data);
		System.out.println("RA: " + IJU.decToSexRA(res[0]) + "\tDEC: " + IJU.decToSexDec(res[1]));

		// c) fits x/y koordinaten
		double fits[] = IJU.ij2fits(img.getHeight(), data);
		System.out.println("FITS X: " + fits[0] + "\tFITS Y: " + fits[1]);
		// d) value on given position
		double imageX = X0;
		double imageY = Y0;
		ImageProcessor ip = img.getProcessor();
		double value = ip.getPixelValue((int) imageX, (int) imageY);
		System.out.println("Value: " + value);
	}

	@Override
	public void mousePressed(MouseEvent e) {
	}

	@Override
	public void mouseReleased(MouseEvent e) {
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}
}
