#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr 13 19:16:13 2023
Plugin CameraTool used to get the Values:
    TEMP_A
    TEMP_B
    TEMP_C
    for the model:   a exp(bx) + c
    to calculate the darkcurrent /pix/s
    The Values must be readout from a given datasheet. Use the list xs/ys this script will then calculate a,b,c.

@author: torsten
"""
import numpy as np
import scipy.optimize
import matplotlib.pyplot as plt

# show statistic
stat=False

### x/y values given from datasheet

#
# BEGIN ADJUST AREA
#

# values for ZWO ASI 2600
xs = np.array([-20.0,-15.0,-10.0,-5.0, 0.0, 5.0, 10.0, 15.0, 20.0, 25.0,30.0])
ys = np.array([0.000123,0.00024,0.00049,0.00099,0.00196,0.00392,0.00781, 0.01563, 0.03126, 0.06252,0.125])
# values for ZWO ASI 178




#
# END ADJUST AREA
#


plt.plot(xs, ys, '.')
plt.title("Original Data")
# =============================================================================
# 
def monoExp(x, m, t, b):
     return m* np.exp(t* x) + b
# perform the fit
p0 = (2000, .1, 50) # start with values near those we expect
params, cv = scipy.optimize.curve_fit(monoExp, xs, ys, p0)
m, t, b = params
sampleRate = 20_000 # Hz
tauSec = (1 / t) / sampleRate
# 
# # determine quality of the fit
squaredDiffs = np.square(ys - monoExp(xs, m, t, b))
squaredDiffsFromMean = np.square(ys - np.mean(ys))
rSquared = 1 - np.sum(squaredDiffs) / np.sum(squaredDiffsFromMean)


if stat:
    print(f"R² = {rSquared}")
    print(f"Y = {m} * e^({t} * x) + {b}")
    print(f"Tau = {tauSec * 1e6} µs")

# 
# plot the results
plt.plot(xs, ys, '.', label="data")
plt.plot(xs, monoExp(xs, m, t, b), '--', label="fitted")
plt.title("DarkCurrent - Fitted Exponential Curve")

print("\n")
print("a exp(bx) + c")
print("--------------")
print("a =",m)
print("b =",t)
print("c =",b)
# =============================================================================
